#include <iostream>
#include <sstream>
#include <iomanip>
#include <string>
#include <stdexcept>
#include <cstdlib>

#include "mahjong.hpp"
#include "util.hpp"

using namespace std;

namespace{
    void warn(const string& msg)
    {
        cerr << msg << endl;
    }

    void error(const string& msg)
    {
        warn(msg);
        exit(1);
    }

    string pai2str(Pai pai)
    {
        ostringstream oss;
        oss << hex << setw(2) << setfill('0') << pai;
        return oss.str();
    }

    string hand2str(const Hand& hand)
    {
        ostringstream oss;
        for(size_t i = 0; i < NELEMS(PAI_LIST); ++i){
            Pai pai = PAI_LIST[i];
            int count = hand.find(pai)->second;
            for(int j = 0; j < count; ++j)
                oss << pai2str(pai);
        }
        return oss.str();
    }

    string result2str(Result result)
    {
        switch(result){
        case RESULT_AGARI: return "2";
        case RESULT_TENPAI: return "1";
        case RESULT_NONE: return "0";
        default: throw logic_error("Unknown result value");
        }
    }

    bool read_hand(const string& s, Hand& hand)
    {
        if(s.size() != 2*14 && s.size() != 2*13) return false;
        int hand_size = s.size()/2;

        hand.clear();
        for(size_t i = 0; i < NELEMS(PAI_LIST); ++i){
            Pai pai = PAI_LIST[i];
            hand.insert(make_pair(pai, 0));
        }

        char buf[3];
        buf[2] = '\0';
        for(int i = 0; i < hand_size; ++i){
            buf[0] = s[2*i];
            buf[1] = s[2*i+1];
            int pai = strtol(buf, 0, 16);
            if(!pai_is_valid(pai)) return false;
            ++hand[pai];
            if(hand[pai] > 4) return false;
        }

        return true;
    }
}

int main(int argc, char** argv)
{
    string line;
    while(getline(cin, line)){
        if(line.empty()) continue;
        if(line[0] == '#') continue;

        Hand hand;
        if(!read_hand(line, hand)){
            warn("Invalid hand: " + line);
            continue;
        }
        cout << hand2str(hand) << "\t" << result2str(hand_chk(hand)) << "\n";
    }

    return 0;
}
